/*
 * Notify log example.
 */

#include "NotifyLogExample.h"


#define LOG_EVENTS 4

NotifyLogExample::NotifyLogExample (CORBA::ORB_var orbVar, ObjectAdapter *pAdapter)
  : POA_CosNotifyComm::PushSupplier (), orbVar_ (orbVar), pAdapter_ (pAdapter), notifyLogFactoryVar_ (0), proxyVar_ (0)
{
  if (CORBA::is_nil (orbVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: ORB not initialised\n")));
      exit (-1);
    }

  // Resolve to the notify log factory.
  CORBA::Object_var objVar = 0;
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Resolving to notify log factory\n")));
  ACE_TRY_NEW_ENV
    {
      objVar = orbVar_->resolve_initial_references (ACE_TEXT ("NotifyLogFactory") ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to resolve to notify log factory\n"));
      exit (-1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to resolve to notify log factory\n")));
      exit (-1);
    }
  ACE_ENDTRY;

  notifyLogFactoryVar_ = DsNotifyLogAdmin::NotifyLogFactory::_narrow (objVar.in ());
  if (CORBA::is_nil (notifyLogFactoryVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Narrowed to a null notify log factory\n")));
      exit (-1);
    }
}
    
NotifyLogExample::~NotifyLogExample ()
{
  // Workaround for the notify log factory having a pure virtual destructor.
  (void) notifyLogFactoryVar_._retn ();
}

DsNotifyLogAdmin::NotifyLog_ptr
NotifyLogExample::create ()
{
  DsNotifyLogAdmin::NotifyLog_var notifyLogVar;
  DsLogAdmin::LogFullActionType logFullAction = DsLogAdmin::wrap;
  CORBA::ULongLong max_size = 0; // 0 means "infinite"
  DsLogAdmin::CapacityAlarmThresholdList_var thresVar = new DsLogAdmin::CapacityAlarmThresholdList ((CORBA::ULong)75);
  CosNotification::QoSProperties qos = CosNotification::QoSProperties (0);
  CosNotification::QoSProperties adm = CosNotification::QoSProperties (0);
  DsLogAdmin::LogId logId = 0;

  ACE_TRY_NEW_ENV
    {
      notifyLogVar = notifyLogFactoryVar_->create (logFullAction, max_size, thresVar.in (), qos, adm, logId ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (DsLogAdmin::InvalidLogFullAction, ilfa)
    {
      ACE_PRINT_EXCEPTION (ilfa, ACE_TEXT ("ERROR: Notify log creation failed with InvalidLogFullAction\n"));
      exit (1);
    }
  ACE_CATCH (DsLogAdmin::InvalidThreshold, it)
    {
      ACE_PRINT_EXCEPTION (it, ACE_TEXT ("ERROR: Notify log creation failed with InvalidThreshold\n"));
      exit (1);
    }
  ACE_CATCH (CosNotification::UnsupportedQoS, uq)
    {
      ACE_PRINT_EXCEPTION (uq, ACE_TEXT ("ERROR: Notify log creation failed with UnsupportedQoS\n"));
      exit (1);
    }
  ACE_CATCH (CosNotification::UnsupportedAdmin, ua)
    {
      ACE_PRINT_EXCEPTION (ua, ACE_TEXT ("ERROR: Notify log creation failed with UnsupportedAdmin\n"));
      exit (1);
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Event log creation failed\n"));
      exit (1);
    }
  ACE_ENDTRY;

  if (CORBA::is_nil (notifyLogVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Notify log is null\n")));
      exit (1);
    }

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Created log with id: %i\n"), logId));
  return notifyLogVar._retn ();
}

void
NotifyLogExample::setFilter (DsNotifyLogAdmin::NotifyLog_ptr channel)
{
  CosNotifyFilter::FilterFactory_var filterFactoryVar = channel->default_filter_factory ();
  CosNotifyFilter::Filter_var filterVar;
  const char *grammar = ACE_TEXT ("EXTENDED_TCL");
  CosNotification::EventTypeSeq evts;
  CosNotifyFilter::ConstraintExpSeq exps;
  CosNotifyFilter::ConstraintExp exp;
  const char *constraint = ACE_TEXT ("$._type_id == 'string'");

  ACE_TRY_NEW_ENV
    {
      // Create the filter.
      filterVar = filterFactoryVar->create_filter (grammar ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      // Set up the constraints.
      evts.length (0);
      exp.event_types = evts;
      exp.constraint_expr = ACE_OS::strdup (constraint);
      exps.length(1);
      exps[0]= exp;
      
      // Add the constraints.
      filterVar->add_constraints (exps ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      // Set the filter.
      channel->set_filter (filterVar.in ());
     }
  ACE_CATCH (CosNotifyFilter::InvalidGrammar, ig)
    {
      ACE_PRINT_EXCEPTION (ig, ACE_TEXT ("ERROR: Failed to create filter -- InvalidGrammar\n"));
      exit (1);
    }
  ACE_CATCH (CosNotifyFilter::InvalidConstraint, ic)
    {
      ACE_PRINT_EXCEPTION (ic, ACE_TEXT ("ERROR: Failed to create filter -- InvalidConstraint\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to create filter\n")));
      exit (1);
    }
  ACE_ENDTRY;

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Filter set on the notify log\n")));
}

void
NotifyLogExample::connect (DsNotifyLogAdmin::NotifyLog_ptr channel)
{
  CORBA::Object_var objVar;
  CosNotifyComm::PushSupplier_var supplierVar;
  CosNotifyChannelAdmin::SupplierAdmin_var adminVar;
  CosNotifyChannelAdmin::ClientType type = CosNotifyChannelAdmin::ANY_EVENT;
  CosNotifyChannelAdmin::ChannelID id;
  CosNotifyChannelAdmin::ProxyConsumer_var pcVar;

  ACE_TRY_NEW_ENV
    {
      // Create the supplier reference.
      objVar = pAdapter_->getPOA ()->servant_to_reference (this);
      supplierVar = CosNotifyComm::PushSupplier::_narrow (objVar.in ());

      // Obtain a proxy.
      adminVar = channel->default_supplier_admin (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
      pcVar = adminVar->obtain_notification_push_consumer (type, id ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      proxyVar_ = CosNotifyChannelAdmin::ProxyPushConsumer::_narrow (pcVar.in ());

      // Perform the connection.
      proxyVar_->connect_any_push_supplier (supplierVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (CosEventChannelAdmin::AlreadyConnected, ac)
    {
      ACE_PRINT_EXCEPTION (ac, ACE_TEXT ("ERROR: Failed to connect -- proxy is already connected\n"));
      exit (1);
    }
  ACE_CATCH (CosNotifyChannelAdmin::AdminLimitExceeded, ale)
    {
      ACE_PRINT_EXCEPTION (ale, ACE_TEXT ("ERROR: Failed to connect -- maximum number of suppliers exceeded\n"));
      exit (1);
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to connect\n"));
      exit (1);
    }
  ACE_ENDTRY;

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Connected supplier to log channel\n")));
}

void
NotifyLogExample::generate ()
{
  DsLogAdmin::Anys data (LOG_EVENTS);
  data.length (LOG_EVENTS);

  double d = 42.424242;
  int n = 123456789;
  char a = 'a';

  data[0] <<= ACE_OS::strdup ("Hello world");
  data[1] <<= d;
  data[2] <<= n;
  data[3] <<= a;
  
  for (int i = 0; i < LOG_EVENTS; i++)
    {
      ACE_TRY_NEW_ENV
        {
          proxyVar_->push (data[i] ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to push message\n"));
          exit (1);
        }
      ACE_CATCHALL
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to push message\n")));
          exit (1);
        }
      ACE_ENDTRY;
    }

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Generated some log messages\n")));
}

void
NotifyLogExample::disconnect ()
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Disconnecting supplier from log channel\n")));

  ACE_TRY_NEW_ENV
    {
      proxyVar_->disconnect_push_consumer (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (CORBA::SystemException, se)
    {
      ACE_PRINT_EXCEPTION (se, ACE_TEXT ("ERROR: Failure while disconnecting\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failure while disconnecting\n")));
      exit (1);
    }
  ACE_ENDTRY;

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Disconnected supplier from log channel\n")));
}

void
NotifyLogExample::disconnect_push_supplier ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Disconnect callback invoked\n")));
}

void
NotifyLogExample::subscription_change (CosNotification::EventTypeSeq const & /*added*/, CosNotification::EventTypeSeq const & /*removed*/)
  ACE_THROW_SPEC ((CosNotifyComm::InvalidEventType,CORBA::SystemException))
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Subscription change callback invoked\n")));
}

int main (int argc, char **argv)
{
  ObjectAdapter *pAdapter;
  ACE_Time_Value *pTV;
  CORBA::ORB_var orbVar;
  NotifyLogExample *pExample;
  DsNotifyLogAdmin::NotifyLog_var logVar;

  // Get the orb reference and start the thread (non-blocking).
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Initialising ORB\n")));
  orbVar = CORBA::ORB_init (argc, argv);
  pTV = new ACE_Time_Value (ACE_Time_Value::zero);
  orbVar->run (pTV);

  // Create the example.
  pAdapter = new ObjectAdapter (orbVar);
  pExample = new NotifyLogExample (orbVar, pAdapter);
  pAdapter->createTransient (pExample);

  // Run the example.
  logVar = pExample->create ();
  pExample->setFilter (logVar.in ());
  pExample->connect (logVar.in ());
  pExample->generate ();
  pExample->disconnect ();

  delete pExample;
  delete pAdapter;

  return 0;
}
