/*
 * Account manager implementation.
 */

#include "AccountManagerImpl.h"


AccountManagerImpl::AccountManagerImpl (Util *pUtil)
  : POA_Bank::AccountManager (), pUtil_ (pUtil)
{
}

void
AccountManagerImpl::createAccount (const char *name, CORBA::Float amount ACE_ENV_ARG_DECL)
  ACE_THROW_SPEC ((CORBA::SystemException, Bank::AccountManager::AlreadyExists))
{
  Bank::Account_var accountVar;
  AccountImpl *pNewAccount;
  PortableServer::POA_var rootPoaVar;
  PortableServer::POAManager_var rootManagerVar;
  CORBA::Object_var objVar;

  ACE_TRY
    {
      accountVar = getAccount (ACE_OS::strdup (name) ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("The account with name %s already exists\n"), name));

      ACE_TRY_THROW (Bank::AccountManager::AlreadyExists ());
    }
  ACE_CATCH (Bank::AccountManager::NotFound, nf)
    {
      // No action needed.
    }
  ACE_ENDTRY;
  ACE_CHECK;

  ACE_TRY
    {
      // Create the implementation for the new account.
      pNewAccount = new AccountImpl (ACE_OS::strdup (name), amount ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      // Create the CORBA object for the new account.
      rootPoaVar = _default_POA ();
      rootManagerVar = rootPoaVar->the_POAManager ();
      rootManagerVar->activate ();
      rootPoaVar->activate_object (pNewAccount);
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Activated account servant\n")));

      // Add the account to the naming service.
      objVar = pNewAccount->_this ();
      pUtil_->addAccount (name, objVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Successfully created account with the following details:\n")));
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Account Holder's name: %s\n"), name));
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Unexpected exception when creating account\n"));
    }
  ACE_ENDTRY;
  ACE_CHECK;
}

Bank::Account_ptr
AccountManagerImpl::getAccount (const char *name ACE_ENV_ARG_DECL)
  ACE_THROW_SPEC ((CORBA::SystemException, Bank::AccountManager::NotFound))
{
  Bank::Account_ptr account;

  account = pUtil_->getAccount (name);

  if (CORBA::is_nil (account))
    {
      ACE_THROW_RETURN (Bank::AccountManager::NotFound (), Bank::Account::_nil ());
    }
  else
    {
      return Bank::Account::_duplicate (account);
    }
}
