/*
 * Server for the naming service bank example.
 */

#include "Naming/Bank/BankS.h"
#include "Naming/Bank/BankC.h"
#include "Naming/Bank/AccountManagerImpl.h"
#include "Naming/Bank/Util.h"

#include <ace/OS.h>
#include <tao/corba.h>


int
main (int argc, char** argv)
{
  CORBA::ORB_var orbVar;
  AccountManagerImpl *pServant;
  CORBA::Object_var servantObjectVar;
  Util *pUtil;
  PortableServer::POA_var poaVar;
  PortableServer::POAManager_var poaManagerVar;
  
  // Get the ORB reference.
  orbVar = CORBA::ORB_init (argc, argv);
  if (CORBA::is_nil (orbVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Could not instantiate the ORB\n")));
      exit (-1);
    }

  // Create the servant.
  pUtil = new Util (orbVar);
  pServant = new AccountManagerImpl (pUtil);
  poaVar = pUtil->getRootPOA ();
  poaManagerVar = poaVar->the_POAManager ();
  poaManagerVar->activate ();
  poaVar->activate_object (pServant);
  servantObjectVar = poaVar->servant_to_reference (pServant);

  // Run the server.
  pUtil->buildNameGraph ();
  pUtil->bindAccountManager (servantObjectVar.in ());
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Waiting to receive incoming requests\n")));
  orbVar->run ();

  // Complete.
  orbVar->destroy ();
  delete pServant;
  delete pUtil;
  return 0;
}
