#include "NamingVideoSupplier.h"


NamingVideoSupplier::NamingVideoSupplier (const char *name , CORBA::Boolean store)
  : VideoSupplier (name, store)
{
  // Note: the base class constructor will add this video supplier to the service provider's collection.
  VideoSupplier::pServiceProvider_->getObjectAdapter ()->addServantToAdapter (this, name, store);
}

NamingVideoSupplier::~NamingVideoSupplier ()
{
}

CORBA::Boolean
NamingVideoSupplier::bind (const CosNaming::NamingContext_ptr inc)
{
  // Declare an arbitary name variable.
  CosNaming::Name name;
  StringArray *array;

  name.length (1);

  ACE_TRY_NEW_ENV
    {
      CosNaming::NamingContext_var typeVar = inc->new_context (ACE_ENV_SINGLE_ARG_PARAMETER);
      name[0].id = ACE_OS::strdup (getType ());
      inc->bind_new_context (name ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHALL
    {
      // OK continue.
    }
  ACE_ENDTRY;

  // Create contexts - ignoring those already existing.
  for (ASA_Iter itb = ASA_Iter (videos_); itb.next (array) != 0; itb.advance ())
    {
      name.length (2);
      ACE_TRY_NEW_ENV
        {
          CosNaming::NamingContext_var typeVar = inc->new_context (ACE_ENV_SINGLE_ARG_PARAMETER);
          name[1].id = ACE_OS::strdup ((*array)[0]);
          inc->bind_new_context (name ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCHALL
        {
          // OK continue.
        }
      ACE_ENDTRY;
    }

  // Bind this company into context.
  name.length (3);

  for (ASA_Iter itb2 = ASA_Iter (videos_); itb2.next (array) != 0; itb2.advance ())
    {
      CORBA::Boolean doRebind = 0;

      name[1].id = ACE_OS::strdup ((*array)[0]);
      name[2].id = ACE_OS::strdup (getName ());
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("%s: %s: %s\n"), getType (), (*array)[0], getName ()));

      ACE_TRY_NEW_ENV
        {
          inc->bind (name, getServant ().in () ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCH (CosNaming::NamingContext::AlreadyBound, ab)
        {
          // Catch already bound and rebind on server re-start.
          doRebind = 1;
        }
      ACE_CATCHALL
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: binding\n")));
          return 0;
        }
      ACE_ENDTRY;
      
      if (doRebind)
        {
          ACE_TRY_NEW_ENV
            {
              ACE_DEBUG ((LM_INFO, ACE_TEXT ("WARNING: rebinding VideoCompany\n")));
              inc->rebind (name, getServant ().in () ACE_ENV_ARG_PARAMETER);
              ACE_TRY_CHECK;
            }
          ACE_CATCHALL
            {
              ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: rebinding\n")));
              return 0;
            }
          ACE_ENDTRY;
        }
    }
          
  return 1;
}

/*
 * @func returns the CORBA reference of this object
 * @rdesc the servant object reference
 */
CORBA::Object_var
NamingVideoSupplier::getServant ()
{
  return VideoSupplier::pServiceProvider_->getObjectAdapter ()->getPOA ()->servant_to_reference (this);
}
