/**
 * Creates structured events representing weather reports and pushes them to
 * an event channel.
 */

#include "WeatherStation.h"


namespace
{
  // All the possible elements of weather information that may go into a report.
  const int MAX_CITIES = 7;
  const int MAX_NEWS   = 2;
  const char * const allNews[MAX_CITIES][MAX_NEWS] =
    {
      { ACE_TEXT ("San Francisco"), ACE_TEXT ("80 miles/hour storm")    } ,
      { ACE_TEXT ("Houston"),       ACE_TEXT ("4 inches of snow")       } ,
      { ACE_TEXT ("New York"),      ACE_TEXT ("Clouds and 43 degrees")  } ,
      { ACE_TEXT ("Seattle"),       ACE_TEXT ("A bit of rain")          } ,
      { ACE_TEXT ("Atlanta"),       ACE_TEXT ("Sun and 89 degrees")     } ,
      { ACE_TEXT ("Newcastle"),     ACE_TEXT ("Rain and mist")          } ,
      { ACE_TEXT ("Louisianna"),    ACE_TEXT ("Twister destroys homes") } 
    };
}

WeatherStation::~WeatherStation ()
{
}

CosNotification::StructuredEvent
WeatherStation::createNextEvent ()
{
  CosNotification::StructuredEvent event;
  const char *city = allNews[rand () % MAX_CITIES][0];
  const char *news = allNews[rand () % MAX_CITIES][1];

  // Event header.
  CosNotification::EventType type;
  type.domain_name = ACE_OS::strdup (ACE_TEXT ("News"));
  type.type_name = ACE_OS::strdup (ACE_TEXT ("Weather"));
  CosNotification::FixedEventHeader fixedHeader;
  fixedHeader.event_type = type;
  fixedHeader.event_name = ACE_OS::strdup (ACE_TEXT ("message"));
  CosNotification::PropertySeq varHeader;
  event.header.fixed_header = fixedHeader;
  event.header.variable_header = varHeader;

  // Filterable data.
  CosNotification::PropertySeq filterable;
  filterable.length (2);
  filterable[0].name = ACE_OS::strdup (ACE_TEXT ("bureau"));
  filterable[0].value <<= ACE_OS::strdup (ACE_TEXT ("Station North"));
  filterable[1].name = ACE_OS::strdup (ACE_TEXT ("city"));
  filterable[1].value <<= ACE_OS::strdup (city);
  event.filterable_data = filterable;

  // Remainder-of-body
  event.remainder_of_body <<= ACE_OS::strdup (news);

  return event;
}

int
main (int argc, char **argv)
{
  // Initialise the ORB instance.
  CORBA::ORB_var orbVar = CORBA::ORB_init (argc, argv, "");

  // Create the weather station.
  WeatherStation weatherStation (orbVar);

  // Connect to the event channel.
  weatherStation.connectToChannel ();

  // Send events.
  weatherStation.broadcast ();

  return 0;
}
