#include "TradingVideoSupplier.h"


TradingVideoSupplier::TradingVideoSupplier (const char *name, const CORBA::Boolean store)
  : VideoSupplier (name, store)
{
  // Note: the base class constructor will add this video supplier to the service provider's collection.
}

TradingVideoSupplier::~TradingVideoSupplier ()
{
}

/*
 * @func returns the CORBA reference of this object
 * @rdesc the servant object reference
 */
CORBA::Object_var
TradingVideoSupplier::getServant ()
{
  return VideoSupplier::pServiceProvider_->getVideoObjectAdapter ()->getPOA ()->servant_to_reference (this);
}

/*
 * @func exports the managed video categories and titles
 * @parm | lookup | reference to trader lookup interface
 */
CORBA::Boolean
TradingVideoSupplier::exportOffer (CosTrading::Lookup_ptr lookup)
{
  StringArray *array;
  char **offer;
  CosTrading::Register_var registerVar;
  CORBA::Object_var objVar;

  // Arbitary video base cost.
  float basePrice = 25.8f;

  registerVar = lookup->register_if ();
  if (CORBA::is_nil (registerVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Null register\n")));
      return 0;
    }

  objVar = getServant ();
  if (CORBA::is_nil (objVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Null servant reference\n")));
      return 0;
    }

  CosTrading::OfferId offer_id;
  CosTrading::PropertySeq properties;
  properties.length (3);

  for (ASA_Iter itb = ASA_Iter (videos_); itb.next (array) != 0; itb.advance ())
    {
      int videos = 0;
      for (StringArray::ITERATOR itb2 = StringArray::ITERATOR (*array); itb2.next (offer) != 0; itb2.advance ())
        {
          // Randomise video cost.
          float inc = (float (rand ()) / RAND_MAX) * 10.0f;
          if ((int) inc > 5)
            {
              inc *= -1.0f;
            }
          basePrice += inc;
          if ((int) basePrice < 10)
            {
              basePrice = 10.0f;
            }

          // Populate offer properties.
          if (videos > 0)
            {
              ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: Category: %s, Title: %s, Cost: %.2f\n"), (*array)[0], (*offer), basePrice));

              // Video category/type.
              CORBA::String_var catVar = ACE_OS::strdup ((*array)[0]);
              properties[0].name = ACE_OS::strdup (ACE_TEXT ("type"));
              properties[0].value <<= catVar;

              // Video title.
              CORBA::String_var titleVar = ACE_OS::strdup ((*offer));
              properties[1].name = ACE_OS::strdup (ACE_TEXT ("title"));
              properties[1].value <<= titleVar;

              // Video cost.
              properties[2].name = ACE_OS::strdup (ACE_TEXT ("cost"));
              properties[2].value <<= basePrice;

              ACE_TRY_NEW_ENV
                {
                  ACE_DEBUG ((LM_INFO, ACE_TEXT ("About to export offers\n")));
                  offer_id = registerVar->_cxx_export (objVar.in (), ACE_TEXT ("IDL:video_supplier:1.0"), properties ACE_ENV_ARG_PARAMETER);
                  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Successfully exported offers\n")));
                }
              ACE_CATCH (CosTrading::UnknownServiceType, ust)
                {
                  ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Unknown type when exporting offer: %s\n"), ust.type.in ()));
                  ACE_PRINT_EXCEPTION (ust, "ERROR: Failed to export offers\n");
                  return 0;
                }
              ACE_CATCHANY
                {
                  ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to export offer\n"));
                  return 0;
                }
              ACE_CATCHALL
                {
                  ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to export offer\n")));
                  return 0;
                }
              ACE_ENDTRY;

              ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: exported: %s\n"), offer_id));
            }
        
          videos++;
        }
    }

  return 1;
}
