#include "VideoServiceProvider.h"


VideoServiceProvider::VideoServiceProvider (CORBA::ORB_var orbVar)
  : orbVar_ (orbVar)
{
  pAdapter_ = new ObjectAdapter (orbVar_, ACE_TEXT ("ServiceProvider"));
  pAdapter_->createTransient ();
  pAdapter_->addServantToAdapter (this, ACE_TEXT ("provider"), 0);

  pAdapterVideos_ = new ObjectAdapter (orbVar_, ACE_TEXT ("Videos"));
}

VideoServiceProvider::~VideoServiceProvider ()
{
  delete pAdapter_;
  delete pAdapterVideos_;
}

/*
 * @func IDL defined funtion that returns a list of companies managed by this service provider
 */
Server::CompanyNameList *
VideoServiceProvider::getCompanies ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Service Provider: managed companies:\n")));

  Server::CompanyNameList *companies = new Server::CompanyNameList;
  companies->length (companies_.size ());
  VideoCompany **company;
  int count = 0;

  for (VideoCompanyArray::ITERATOR itb = VideoCompanyArray::ITERATOR (companies_); itb.next (company) != 0; itb.advance ())
    {
      (*companies)[count] = ACE_OS::strdup ((*company)->getName ());
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("%s\n"), (*company)->getName ()));
      count++;
    }

  return companies;
}

void
VideoServiceProvider::store (VideoCompany *company, const char *name)
{
  if (companies_.size () == 0)
    {
      pAdapterVideos_->createTransient (company);
    }
  else
    {
      pAdapterVideos_->addServantToAdapter (company, name, 0);
    }

  companies_.size (companies_.size () + 1);
  companies_[companies_.size () - 1] = company;
}

void
VideoServiceProvider::activateManager ()
{
  pAdapter_->activateManager ();
  pAdapterVideos_->activateManager ();
}

CORBA::Object_var
VideoServiceProvider::getServant ()
{
  return pAdapter_->getPOA ()->servant_to_reference (this);
}

void
VideoServiceProvider::helloWorld ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Hello World\n")));
}

#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
template class ACE_Array<VideoCompany*>;
template class ACE_Array_Base<VideoCompany*>;
template class ACE_Array_Iterator<VideoCompany*>;
#endif // ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION
