/*
 * This program is a client application that resolves a reference to the trading service
 * and performs simple Trading Service Object look-ups based on the static properties describing
 * video-on-demand suppliers.
 */

#include <CosTradingC.h>

#include "Common/ServerC.h"

#include <tao/corba.h>
#include <ace/OS.h>


int
main (int argc, char **argv)
{
  CORBA::ORB_var orbVar;
  CORBA::Object_var objVar;
  CosTrading::Lookup_var lookupVar;

  // Initialise the ORB.
  ACE_TRY_NEW_ENV
    {
      orbVar = CORBA::ORB_init (argc, argv ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: Initialised ORB\n")));
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Cannot initialise ORB\n"));
      exit (-1);
    }
  ACE_ENDTRY;

  // Get a reference to the trading service.
  ACE_TRY_NEW_ENV
    {
      objVar = orbVar->resolve_initial_references (ACE_TEXT ("TradingService") ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      lookupVar = CosTrading::Lookup::_narrow (objVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to resolve/narrow to trading service\n"));
      exit (-1);
    }
  ACE_ENDTRY;

  // Set up an empty policy and execute the query.

  CosTrading::PolicySeq_var policiesVar = new CosTrading::PolicySeq ();
  CosTrading::Lookup::SpecifiedProps desiredProps;
  CosTrading::PropertyNameSeq_var propertySeqVar = new CosTrading::PropertyNameSeq ();
  propertySeqVar->length (1);
  propertySeqVar[(CORBA::ULong)0] = ACE_OS::strdup (ACE_TEXT ("title"));
  desiredProps.prop_names (propertySeqVar.in ());

  CosTrading::OfferSeq_var offersVar;
  CosTrading::PolicyNameSeq_var policiesAppliedVar;
  CosTrading::OfferIterator_var iteratorVar;

  ACE_TRY_NEW_ENV
    {
      lookupVar->query (ACE_TEXT ("IDL:video_supplier:1.0"),
                        ACE_TEXT (""),
                        ACE_TEXT (""),
                        policiesVar,
                        desiredProps,
                        0,
                        offersVar,
                        iteratorVar,
                        policiesAppliedVar
                        ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to execute the query\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to execute the query\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Process the results.
   
  if (!CORBA::is_nil (iteratorVar.in ()))
    {
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Trader lookup for all references\n")));

      CORBA::Boolean more;
      int count1 = 0;
      do
        {
          more = iteratorVar->next_n (10, offersVar);
          if (more)
            {
              ACE_DEBUG ((LM_INFO, ACE_TEXT ("%i available titles.\n"), offersVar->length ()));
              ACE_DEBUG ((LM_INFO, ACE_TEXT ("===================\n")));
              for (CORBA::ULong count2 = 0; count2 < offersVar->length (); count2++)
                {
                  // Print the information for each offer.
                  const char *temp = 0;
                  (*offersVar)[count2].properties[0].value >>= temp;
                  CORBA::String_var titleVar (temp);
                  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Title: %s\n"), titleVar.in ()));
                }
            }
          else
            {
              if ( count1 == 0)
                {
                  ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: No offers found\n")));
                }
              else
                {
                  ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: No more offers found\n")));
                }
            }

            count1++;
        } while (more);

      iteratorVar->destroy ();
    }
  else
    {
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("WARNING: No iterator returned from query\n")));
    }

  // Complete.
  return 0;
}
