// $Id: SSLExample_i.cpp,v 1.3 2007-03-20 13:13:00 tao Exp $

#include "SSLExample_i.h"


int
main (int argc, char *argv[])
{
  try
    {
      SSLExample_i impl (argc, argv);

      if (impl.test_)
        {
          ACE_DEBUG ((LM_DEBUG, "Running in automated test mode\n"));
          impl.start_test ();
        }
      else
        {
          impl.handle_user_input ();
        }
    }
  catch (CORBA::SystemException &se)
    {
      ACE_PRINT_EXCEPTION (se, "Error while running\n");
      return 1;
    }
  catch (...)
    {
      ACE_DEBUG ((LM_ERROR, "Error while running\n"));
      return 1;
    }

  // Delay before exit.
  ACE_OS::sleep (5);

  return 0;
}


SSLExample_i::SSLExample_i (int argc, char *argv[])
  : test_ (false), orbVar_ (0), rootPoaVar_ (0), poaManagerVar_ (0), currentVar_ (0), orb_thread_ (0),
    server_ior_file_ ("server.ior"), client_ior_file_ ("client.ior"), service_ior_file_ ("service.ior"),
    clientVar_ (0), serverVar_ (0), serviceVar_ (0),
    is_server_ (false), is_client_ (false), is_service_ (false),
    SERVER_STRING ("server "), CLIENT_STRING ("client "), SERVICE_STRING ("service "), EXIT_STRING ("exit"),
    ior_file_ptr_ (0), role_name_ (0)
{
  initialise (argc, argv);
  
  // Create the CORBA object.
  try
    {
      SSLExample_var thisVar = _this ();
      CORBA::String_var iorVar = orbVar_->object_to_string (thisVar.in ());
      ACE_DEBUG ((LM_DEBUG, "Activated %s as %s\n", role_name_, iorVar.in ()));

      // Write out the IOR.
      writeIOR (iorVar.in (), *ior_file_ptr_);
    }
  catch (CORBA::SystemException &se)
    {
      ACE_PRINT_EXCEPTION (se, "Failed to create the instance\n");
      exit (1);
    }
}


SSLExample_i::~SSLExample_i ()
{
  // Ensure the STDIN handler is removed.
  if (! test_)
    {
      if (ACE_Event_Handler::remove_stdin_handler (TAO_ORB_Core_instance ()->reactor (),
                                                   TAO_ORB_Core_instance ()->thr_mgr ()) == -1)
        {
#if !defined (ACE_WIN32)
          ACE_DEBUG ((LM_ERROR, "Failed to remove the STDIN handler\n"));
#endif
        }
    }

  if (! orb_thread_)
    {
      delete orb_thread_;
    }
}


void
SSLExample_i::send_message (const char *message)
  throw (CORBA::SystemException)
{
  ACE_DEBUG ((LM_DEBUG, "Received message: %s", message));

  try
    {
      if (! currentVar_)
        {
          ACE_DEBUG ((LM_DEBUG, " (cannot determine: no SSLIOPCurrent reference)\n"));
        }
      else
        {
          SSLIOP::ASN_1_Cert_var cert = currentVar_->get_peer_certificate ();
          ACE_DEBUG ((LM_DEBUG, " (invocation via SSLIOP)\n"));
        }
    }
  catch (SSLIOP::Current::NoContext &)
    {
      ACE_DEBUG ((LM_DEBUG, " (invocation via IIOP)\n"));
    }
  catch (CORBA::SystemException &se)
    {
      ACE_PRINT_EXCEPTION (se, "\nFailed to query SSLIOP::Current\n");
    }
  catch (...)
    {
      ACE_DEBUG ((LM_ERROR, "\nFailed to query SSLIOP::Current\n"));
    }
}


void
SSLExample_i::start_test ()
  throw ()
{
  char line[100];

  // Delay to allow all components to initialise.
  ACE_DEBUG ((LM_DEBUG, "Delaying before test start\n"));
  ACE_OS::sleep (30);

  // Loop, sending to the other parts, with a small delay between iterations.
  for (int i = 1 ; i <= 10 ; i++)
    {
      if (is_server_)
        {
          ACE_OS::sprintf (line, "%sMessage #%i from %s", CLIENT_STRING, i, role_name_);
          parse_line (line);
          ACE_OS::sprintf (line, "%sMessage #%i from %s", SERVICE_STRING, i, role_name_);
          parse_line (line);
        }
      else if (is_client_)
        {
          ACE_OS::sprintf (line, "%sMessage #%i from %s", SERVER_STRING, i, role_name_);
          parse_line (line);
          ACE_OS::sprintf (line, "%sMessage #%i from %s", SERVICE_STRING, i, role_name_);
          parse_line (line);
        }

      ACE_OS::sleep (1);
    }

  // Delay to allow any incoming calls to complete.
  ACE_DEBUG ((LM_DEBUG, "Delaying before test exit\n"));
  ACE_OS::sleep (30);

  // Shutdown the ORB.
  orbVar_->shutdown ();
}


void
SSLExample_i::handle_user_input ()
{
  // Register the STDIN handler.
  if (ACE_Event_Handler::register_stdin_handler (this, TAO_ORB_Core_instance ()->reactor (), TAO_ORB_Core_instance ()->thr_mgr ()) == -1)
    {
      ACE_DEBUG ((LM_ERROR, "Failed to register the STDIN handler\n"));
      exit (1);
    }

  // Display instructions.
  ACE_DEBUG ((LM_DEBUG, "Ready: waiting for user input, \"%s\" to exit\n", EXIT_STRING));
  if (is_server_)
    {
      ACE_DEBUG ((LM_DEBUG, "%s xxxx       Send string xxxx to client\n", CLIENT_STRING));
    }
  if (is_client_)
    {
      ACE_DEBUG ((LM_DEBUG, "%s xxxx       Send string xxxx to server\n", SERVER_STRING));
    }
  if (is_server_ || is_client_)
    {
      ACE_DEBUG ((LM_DEBUG, "%s xxxx      Send string xxxx to service\n", SERVICE_STRING));
    }

  // Wait on the ORB thread.
  orb_thread_->thr_mgr ()->wait ();
}


int
SSLExample_i::parse_args (int argc, char *argv[])
{
  ACE_Get_Opt get_opts (argc, argv, "s:c:n:r:t");
  int c;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 's':
        server_ior_file_ = get_opts.opt_arg ();
        ACE_DEBUG ((LM_DEBUG, "Server IOR file: %s\n", server_ior_file_));
        break;
      case 'c':
        client_ior_file_ = get_opts.opt_arg ();
        ACE_DEBUG ((LM_DEBUG, "Client IOR file: %s\n", client_ior_file_));
        break;
      case 'n':
        service_ior_file_ = get_opts.opt_arg ();
        ACE_DEBUG ((LM_DEBUG, "Generic service IOR file: %s\n", service_ior_file_));
        break;
      case 'r':
        role_name_ = get_opts.opt_arg ();
        if (! ACE_OS::strcmp (role_name_, "server"))
          {
            is_server_ = true;
            ior_file_ptr_ = &server_ior_file_;
          }
        else if (! ACE_OS::strcmp (role_name_, "client"))
          {
            is_client_ = true;
            ior_file_ptr_ = &client_ior_file_;
          }
        else if (! ACE_OS::strcmp (role_name_, "service"))
          {
            is_service_ = true;
            ior_file_ptr_ = &service_ior_file_;
          }
        break;
      case 't':
        test_ = true;
        break;
      case '?':
      default:
        ACE_DEBUG ((LM_DEBUG, "Usage:  %s [-s server_ior_file] [-c client_ior_file] [-n service_ior_file] -r client|server|service\n", argv [0]));
        return -1;
      }

  return 0;
}


void
SSLExample_i::initialise (int argc, char *argv[])
{
  // Initialise the ORB and obtain/activate the root POA.
  try
    {
      orbVar_ = CORBA::ORB_init (argc, argv, "");
      CORBA::Object_var objVar = orbVar_->resolve_initial_references ("RootPOA");
      if (CORBA::is_nil (objVar.in ()))
        {
          ACE_DEBUG ((LM_ERROR, "Unable to resolve to the root POA\n"));
          exit (-1);
        }
      rootPoaVar_ = PortableServer::POA::_narrow (objVar.in ());
      poaManagerVar_ = rootPoaVar_->the_POAManager ();
      poaManagerVar_->activate ();
    }
  catch (CORBA::SystemException &se)
    {
      ACE_PRINT_EXCEPTION (se, "Failed to initialise\n");
      exit (-1);
    }

  // Start the ORB thread.
  orb_thread_ = new ORBThread (orbVar_.in ());
  int ret = orb_thread_->activate (THR_NEW_LWP | THR_JOINABLE, 1);
  if (ret != 0)
    {
      ACE_DEBUG ((LM_ERROR, "Failed to activate the ORB thread (%i)\n", ret));
      exit (-1);
    }

  // Obtain a reference to the SSLIOP::Current object.  This will not be present
  // if not running under SSL.
  try
    {
      ACE_DEBUG ((LM_DEBUG, "Resolving to SSLIOPCurrent...\n"));
      CORBA::Object_var objVar = orbVar_->resolve_initial_references ("SSLIOPCurrent");
      if (CORBA::is_nil (objVar.in ()))
        {
          ACE_DEBUG ((LM_ERROR, "Unable to resolve to the SSLIOP Current (nil object returned)\n"));
          currentVar_ = 0;
        }
      else
        {
          currentVar_ = SSLIOP::Current::_narrow (objVar.in ());
        }
    }
  catch (CORBA::ORB::InvalidName &)
    {
      ACE_DEBUG ((LM_ERROR, "Unable to resolve to the SSLIOP Current (InvalidName)\n"));
      currentVar_ = 0;
    }
  catch (CORBA::SystemException &se)
    {
      ACE_PRINT_EXCEPTION (se, "Unexpected error when obtaining reference to the SSLIP Current\n");
      exit (-1);
    }

  // Parse the command line arguments for the IOR file names.
  if (parse_args (argc, argv) != 0)
    {
      ACE_DEBUG ((LM_ERROR, "Failed to parse command line arguments\n"));
      exit (-1);
    }

  // Confirm that the role was established.
  if (! is_client_ && ! is_server_ && ! is_service_)
    {
      ACE_DEBUG ((LM_ERROR, "Role was not specified or was not recognised (must be either client, server or service)\n"));
      exit (-1);
    }
}


char *
SSLExample_i::readIOR (const char *filename)
{
  ACE_HANDLE f_handle = ACE_OS::open (filename, 0);
  if (f_handle == ACE_INVALID_HANDLE)
    {
      ACE_DEBUG ((LM_ERROR, "Cannot open input file for reading IOR (invalid handle): %s\n", filename));
      return 0;
    }

  ACE_Read_Buffer ior_buffer (f_handle);
  char *data = ior_buffer.read ();
  if (data == 0)
    {
      ACE_DEBUG ((LM_ERROR, "Unable to read IOR from the input file: %s\n", filename));
      return 0;
    }

  char *ior = ACE_OS::strdup (data);
  ior_buffer.alloc ()->free (data);
  ACE_OS::close (f_handle);
  
  return ior;
}


int
SSLExample_i::writeIOR (const char *ior, const char *filename)
{
  FILE *output_file= ACE_OS::fopen (filename, "w");
  if (output_file == 0)
    {
      ACE_DEBUG ((LM_ERROR, "Cannot open output file for writing IOR: %s\n", filename));
      return -1;
    }

  ACE_OS::fprintf (output_file, "%s", ior);
  ACE_OS::fclose (output_file);
  return 0;
}


int
SSLExample_i::handle_input (ACE_HANDLE)
{
  char buf[BUFSIZ];

  if (ACE_OS::fgets (buf, BUFSIZ, stdin) == 0)
    {
      return 0;
    }

  // Strip the carriage return / line feed characters.
  buf[ACE_OS::strcspn (buf, "\n")] = '\0';

  try
    {
      parse_line (buf);
    }
  catch (CORBA::SystemException &se)
    {
      ACE_PRINT_EXCEPTION (se, "Failed to activate / send to target\n");
    }
  catch (...)
    {
      ACE_DEBUG ((LM_ERROR, "Failed to activate / send to target\n"));
    }

  return 0;
}


void
SSLExample_i::parse_line (char *line)
  throw ()
{
  // Note: we delay obtaining a reference to the other components until the first use,
  // so that all can be started and write out their IORs without trying to read the IORs
  // from the other two.

  if (! ACE_OS::strcmp (line, EXIT_STRING))
    {
      // Shutdown the ORB.
      ACE_DEBUG ((LM_DEBUG, "Shutting down ORB\n"));
      orbVar_->shutdown (true);
    }
  else if (! ACE_OS::strncmp (line, SERVER_STRING, ACE_OS::strlen (SERVER_STRING)) && is_client_)
    {
      char *message = ACE_OS::strdup (line + ACE_OS::strlen (SERVER_STRING));

      if (serverVar_ == 0)
        {
          ACE_DEBUG ((LM_DEBUG, "Obtaining reference to server\n"));
          char *ior = readIOR (server_ior_file_);
          CORBA::Object_var objVar = orbVar_->string_to_object (ior);
          serverVar_ = SSLExample::_narrow (objVar.in ());
        }
                
      ACE_DEBUG ((LM_DEBUG, "Calling server with message: %s\n", message));
      serverVar_->send_message (message);
    }
  else if (! ACE_OS::strncmp (line, CLIENT_STRING, ACE_OS::strlen (CLIENT_STRING)) && is_server_)
    {
      char *message = ACE_OS::strdup (line + ACE_OS::strlen (CLIENT_STRING));

      if (clientVar_ == 0)
        {
          ACE_DEBUG ((LM_DEBUG, "Obtaining reference to client\n"));
          char *ior = readIOR (client_ior_file_);
          CORBA::Object_var objVar = orbVar_->string_to_object (ior);
          clientVar_ = SSLExample::_narrow (objVar.in ());
        }
                
      ACE_DEBUG ((LM_DEBUG, "Calling client with message: %s\n", message));
      clientVar_->send_message (message);
    }
  else if (! ACE_OS::strncmp (line, SERVICE_STRING, ACE_OS::strlen (SERVICE_STRING)) && ! is_service_)
    {
      char *message = ACE_OS::strdup (line + ACE_OS::strlen (SERVICE_STRING));

      if (serviceVar_ == 0)
        {
          ACE_DEBUG ((LM_DEBUG, "Obtaining reference to service\n"));
          char *ior = readIOR (service_ior_file_);
          CORBA::Object_var objVar = orbVar_->string_to_object (ior);
          serviceVar_ = SSLExample::_narrow (objVar.in ());
        }
                
      ACE_DEBUG ((LM_DEBUG, "Calling service with message: %s\n", message));
      serviceVar_->send_message (message);
    }
}


ORBThread::ORBThread (CORBA::ORB_ptr orb)
  : orbVar_ (CORBA::ORB::_duplicate (orb))
{
}


int
ORBThread::svc (void)
{
  try
    {
      this->orbVar_->run ();
    }
  catch (CORBA::SystemException &se)
    {
      ACE_PRINT_EXCEPTION (se, "Failed to run the ORB\n");
    }
  catch (...)
    {
      ACE_DEBUG ((LM_ERROR, "Failed to run the ORB\n"));
    }

  return 0;
}
