// -*- C++ -*-

//=============================================================================
/**
 *  @file    NodeApplicationManager_Impl.h
 *
 *  $Id: NodeApplicationManager_Impl.h,v 1.1.1.1 2007-06-22 14:15:07 sm Exp $
 *
 * @Brief  Implementation of POA_Deployment::NodeApplicationManager
 *
 * @author Erwin Gottlieb <eg@prismtech.com>
 */
//=============================================================================

#ifndef NODEAPPLICATIONMANAGER_IMPL_H_
#define NODEAPPLICATIONMANAGER_IMPL_H_

#include "NodeApplicationManager_Export.h"
#include "Deployment/Deployment_DeploymentPlanC.h"
#include "Deployment/Deployment_NodeApplicationManagerS.h"
#include "Deployment/Deployment_NodeApplicationC.h"
#include "NodeApplication/NodeApplication_Impl.h"
#include "RedirectionService/RedirectionService.h"
#include "tao/Utils/Servant_Var.h"

namespace DAnCE
  {

  class NodeManager_Impl;

  class NodeApplicationManager_Export NodeApplicationManager_Impl : public virtual POA_Deployment::NodeApplicationManager
    {
    public:
      NodeApplicationManager_Impl (CORBA::ORB_ptr orb
                                   , PortableServer::POA_ptr poa
                                   , const Deployment::DeploymentPlan& plan
                                   , RedirectionService & redirection
                                   , const ACE_CString& name
                                   , CORBA::Boolean ignore_failures
                                   , TimeBase::TimeT ignore_failures_timeout);

      virtual ~NodeApplicationManager_Impl();

      virtual Deployment::Application_ptr
      startLaunch (const Deployment::Properties & configProperty,
                   Deployment::Connections_out providedReference)
      ACE_THROW_SPEC ( (
                         ::CORBA::SystemException
                         , ::Deployment::ResourceNotAvailable
                         , ::Deployment::StartError
                         , ::Deployment::InvalidProperty
                         , ::Deployment::InvalidNodeExecParameter
                         , ::Deployment::InvalidComponentExecParameter
                       ));

      virtual void destroyApplication (Deployment::Application_ptr)
      ACE_THROW_SPEC ( (
                         ::CORBA::SystemException
                         , ::Deployment::StopError
                       ));
    private:
      void register_plan();

      Deployment::DeploymentPlan plan_;
      CORBA::ORB_var orb_;
      PortableServer::POA_var poa_;
      typedef TAO::Utils::Servant_Var<NodeApplication_Impl> NodeApplicationImpl_var;
      NodeApplicationImpl_var application_;
      RedirectionService & redirection_;
      ACE_CString node_name_;
      CORBA::Boolean ignore_failures_;
      TimeBase::TimeT ignore_failures_timeout_;
    };
};
#endif /*NODEAPPLICATIONMANAGER_IMPL_H_*/
