// -*- C++ -*-

//=============================================================================
/**
 *  @file POAFiltering.h
 *
 *  $Id: POAFiltering.h,v 1.1 2007-06-06 11:07:59 vz Exp $
 *
 *  Dynamic loader object for POA Filtering
 *
 *  @author  Vladimir Zykov <Vladimir.Zykov@prismtech.com>
 */
//=============================================================================

#ifndef TAO_POAFILTERING_H
#define TAO_POAFILTERING_H
#include /**/ "ace/pre.h"

#include "tao/POAFiltering/poafiltering_export.h"

#if !defined (ACE_LACKS_PRAGMA_ONCE)
# pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

#define TAO_POAFILTERING_SAFE_INCLUDE
#include "tao/POAFiltering/POAFilterPolicyC.h"
#undef TAO_POAFILTERING_SAFE_INCLUDE

#include "tao/PortableServer/POAFilterCreator_Adapter.h"
#include "ace/Service_Config.h"

TAO_BEGIN_VERSIONED_NAMESPACE_DECL

class TAO_ORB_Core;
class TAO_Root_POA;
class TAO_Acceptor_Filter;

/**
 * @class TAO_POAFiltering_Loader
 *
 * @brief Class that loads the POAFiltering library.
 */

class TAO_POAFiltering_Export TAO_POAFiltering_Loader
  : public TAO_POAFilterCreator_Adapter
{
public:

  /// Constructor
  TAO_POAFiltering_Loader (void);

  /// Destructor
  virtual ~TAO_POAFiltering_Loader (void);

  /// Initialize the POAFiltering loader hooks.
  virtual int init (int argc,
                    ACE_TCHAR* []);

  /// Create instance of TAO_POA_Acceptor_Filter.
  virtual TAO_Acceptor_Filter *create_acceptor_filter (TAO_ORB_Core &oc,
                                                       TAO_Root_POA &poa);

  /// Used to force the initialization of the ORB code.
  static int Initializer (void);

private:

  /// Flag to indicate whether the POAFiltering library has been
  /// activated.
  static int is_activated_;
};

static int
TAO_Requires_POAFiltering_Initializer = TAO_POAFiltering_Loader::Initializer ();

TAO_END_VERSIONED_NAMESPACE_DECL

ACE_STATIC_SVC_DECLARE (TAO_POAFiltering_Loader)
ACE_FACTORY_DECLARE (TAO_POAFiltering, TAO_POAFiltering_Loader)


#include /**/ "ace/post.h"
#endif /* TAO_POAFILTERING_H */
