// @(#) $Id: POA_Acceptor_Filter.cpp,v 1.4 2007-06-19 12:34:51 vz Exp $

#include "tao/POAFiltering/POA_Acceptor_Filter.h"
#include "tao/PortableServer/Root_POA.h"
#include "tao/Transport_Acceptor.h"
#include "tao/MProfile.h"
#include "tao/Profile.h"
#include "ace/INET_Addr.h"

ACE_RCSID(POAFiltering,
          POA_Acceptor_Filter,
          "$Id: POA_Acceptor_Filter.cpp,v 1.4 2007-06-19 12:34:51 vz Exp $")

TAO_BEGIN_VERSIONED_NAMESPACE_DECL

TAO_POA_Acceptor_Filter::TAO_POA_Acceptor_Filter (TAO_ORB_Core &oc,
                                                  TAO_Root_POA &poa)
  : orb_core_ (oc)
  , poa_ (poa)
{
}

int
TAO_POA_Acceptor_Filter::fill_profile (const TAO::ObjectKey &object_key,
                                       TAO_MProfile &mprofile,
                                       TAO_Acceptor **acceptors_begin,
                                       TAO_Acceptor **acceptors_end,
                                       CORBA::Short priority)
{
  // Go through all the acceptors.
  for (TAO_Acceptor** acceptor = acceptors_begin;
       acceptor != acceptors_end;
       ++acceptor)
    {
      // Ask each acceptor to make a profile.
      if ((*acceptor)->create_profile (object_key,
                                       mprofile,
                                       priority) == -1)
        return -1;
    }

  CORBA::String_var poa_name = this->poa_.the_name ();
  TAO_EndpointSet poa_endpoints;

  this->orb_core_.orb_params ()->get_poa_endpoint_set (
    ACE_CString (poa_name.in ()), poa_endpoints);

  if (!poa_endpoints.is_empty ())
    {
      for (CORBA::ULong i = 0;
           i < mprofile.profile_count ();
           ++i)
        {
          TAO_Profile *profile = mprofile.get_profile (i);

          if (this->is_profile_filtered (*profile, poa_endpoints))
            {
              mprofile.remove_profile (profile);
              // And after removing profile one step back.
              --i;
            } 
        }
    }

  return 0;
}

int
TAO_POA_Acceptor_Filter::encode_endpoints (TAO_MProfile &mprofile)
{
  // if -ORBUseSharedProfile is set, there may be multiple endpoints
  // per profile, even without priority.
  for (CORBA::ULong i = 0;
       i < mprofile.profile_count ();
       ++i)
    {
      TAO_Profile *profile = mprofile.get_profile (i);
      if (profile->encode_alternate_endpoints () == -1)
        return -1;
    }

  return 0;
}

bool
TAO_POA_Acceptor_Filter::is_profile_filtered (
  TAO_Profile &profile,
  const TAO_EndpointSet &poa_endpoints) const
{
  ACE_CString p_protocol;
  TAO_EndpointSet p_endpoints;
  this->parse_corbaloc_address (profile, p_protocol, p_endpoints);

  TAO_EndpointSetIterator p_endpts (p_endpoints);
  for (ACE_CString *p_ep = 0;
       p_endpts.next (p_ep) != 0;
       p_endpts.advance ())
    {
      const ACE_CString &p_addr = (*p_ep);

      TAO_EndpointSetIterator poa_endpts (poa_endpoints);
      for (ACE_CString *poa_ep = 0;
           poa_endpts.next (poa_ep) != 0;
           poa_endpts.advance ())
        {
          const ACE_CString &iop = (*poa_ep);

          ACE_CString r_protocol;
          TAO_EndpointSet r_endpoints;
          this->parse_url_address (iop.c_str (),
                                   r_protocol,
                                   r_endpoints);

          // Protocols must be the same.
          if (p_protocol != r_protocol)
            continue;

          TAO_EndpointSetIterator r_endpts (r_endpoints);
          for (ACE_CString *r_ep = 0;
               r_endpts.next (r_ep) != 0;
               r_endpts.advance ())
            {
              const ACE_CString &r_addr = (*r_ep);

              // Maybe we are lucky and don't need to do ugly things.
              if (p_addr == r_addr)
                return false;

              // Well, we are not lucky and in this case we do a lot
              // of assumptions:
              // * the most important we assume that we work with IP addresses.
              ACE_INET_Addr r_ip_addr;
              if (r_ip_addr.set (r_addr.c_str ()) == 0)
                {
                  ACE_INET_Addr p_ip_addr (p_addr.c_str ());
                  if (r_ip_addr == p_ip_addr ||
                      r_ip_addr.is_any () &&
                      r_ip_addr.get_port_number () ==
                        p_ip_addr.get_port_number ())
                    return false;
                }

              // We do still try and do even uglier things.
              // * we assume that address contains ':';
              // * if there is nothing before or after ':' in iop_addr
              //   then we only compare the existing part to corresponding
              //   part in addr.
              ACE_CString p_port, r_port;
              ACE_CString p_tmp_addr = p_addr;
              ACE_CString r_tmp_addr = r_addr;
              const char *p_addr_pos = p_tmp_addr.c_str ();
              const char *r_addr_pos = r_tmp_addr.c_str ();
              const char *end = 0;
#if defined (ACE_HAS_IPV6)
              if (r_addr_pos[0] == '[')
                {
                  if (p_addr_pos[0] != '[')
                    continue;

                  end = ACE_OS::strchr (r_addr_pos, ']');
                  if (end == 0) // maybe it's an exception
                    continue;
                  r_port.set (end + 1); // Even if port contains ':' it's not
                  // a problem since port from profile will contain it too.
                  if (end != r_addr_pos)
                    r_tmp_addr = r_tmp_addr.substr (0, end + 1 - r_addr_pos);
                  else
                    r_tmp_addr.fast_clear ();

                  end = ACE_OS::strchr (p_addr_pos, ']');
                  p_port.set (end + 1);
                  p_tmp_addr = p_tmp_addr.substr (0, end + 1 - p_addr_pos);
                }
              else
#endif /* ACE_HAS_IPV6 */
                {
                  end = ACE_OS::strchr (r_addr_pos, ':');
                  if (end == 0)
                    continue;
                  r_port.set (end + 1);
                  if (end != r_addr_pos)
                    r_tmp_addr = r_tmp_addr.substr (0, end - r_addr_pos);
                  else
                    r_tmp_addr.fast_clear ();

#if defined (ACE_HAS_IPV6)
                  if (p_addr_pos[0] == '[')
                    {
                      end = ACE_OS::strchr (p_addr_pos, ']');
                      p_port.set (end + 2);
                      p_tmp_addr = p_tmp_addr.substr (0, end + 1 - p_addr_pos);
                    }
                  else
#endif /* ACE_HAS_IPV6 */
                    {
                      end = ACE_OS::strchr (p_addr_pos, ':');
                      p_port.set (end + 1);
                      p_tmp_addr = p_tmp_addr.substr (0, end - p_addr_pos);
                    }
                }

              if (r_tmp_addr.length () == 0 &&
                  p_port == r_port ||
                  r_port.length () == 0 &&
                  p_tmp_addr == r_tmp_addr)
                return false;
            }
        }
    }

  return true;
}

int
TAO_POA_Acceptor_Filter::parse_url_address (
  const char *ior,
  ACE_CString &protocol,
  TAO_EndpointSet &endpoints) const
{
  const char *end = ACE_OS::strchr (ior, ':');
  protocol.set (ior, end - ior, 1);
  // '://' is always present since it's checked in
  // TAO_ORB_Parameters::parse_and_add_endpoints

  ior = end + 3; // 3 for '://'

  while (true)
    {
      // We ignore completely the version specifiers.
      if (isdigit (ior [0]) &&
          ior[1] == '.' &&
          isdigit (ior [2]) &&
          ior[3] == '@')
        ior += 4;

      end = ACE_OS::strchr (ior, ',');
      if (end)
        {
          ACE_CString addr (ior, end - ior);
          endpoints.enqueue_tail (addr);

          ior = end + 1;
        }
      else
        {
          ACE_CString addr (ior);
          endpoints.enqueue_tail (addr);
          break;
        }
    }

  return 0;
}

int
TAO_POA_Acceptor_Filter::parse_corbaloc_address (
  TAO_Profile &profile,
  ACE_CString &protocol,
  TAO_EndpointSet &endpoints) const
{
  CORBA::String_var ior = profile.to_string ();

  const char corbaloc[] = "corbaloc:";
  const char *pos = ACE_OS::strstr (ior.in (), corbaloc);
  pos += sizeof (corbaloc) - 1;

  // We believe unconditionally that corbaloc syntax is ok.
  protocol.set (pos, ACE_OS::strchr (pos, ':') - pos, 1);

  const char *delim = ACE_OS::strchr (pos, profile.object_key_delimiter ());
  while (true)
    {
      const char *end = ACE_OS::strchr (pos, ':');

      pos = end + 1 + 4; // 1 for ':' and 4 for 'V.V@'; the later is
      // added to corbaloc by all known profile implementations.

      end = ACE_OS::strchr (pos, ',');
      // ObjectKey can contain comma too. The more reliable check
      // for multiple endpoints in profile is to call endpoint_count
      if (end && end < delim)
        {
          ACE_CString addr (pos, end - pos);
          endpoints.enqueue_tail (addr);

          pos = end + 1;
        }
      else
        {
          ACE_CString addr (pos, delim - pos);
          endpoints.enqueue_tail (addr);
          break;
        }
    }

  return 0;
}

TAO_END_VERSIONED_NAMESPACE_DECL
