// -*- C++ -*-

//=============================================================================
/**
 *  @file    POA_Acceptor_Filter.h
 *
 *  $Id: POA_Acceptor_Filter.h,v 1.1 2007-06-06 11:07:59 vz Exp $
 *
 *  A strategy with filtering of profiles.
 *
 *  @author  Vladimir Zykov <vz@prismtech.com>
 */
//=============================================================================

#ifndef TAO_POA_ACCEPTOR_FILTER_H
#define TAO_POA_ACCEPTOR_FILTER_H

#include /**/ "ace/pre.h"

#include "tao/POAFiltering/poafiltering_export.h"

#if !defined (ACE_LACKS_PRAGMA_ONCE)
# pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

#include "tao/Acceptor_Filter.h"
#include "tao/ORB_Core.h"
#include "tao/params.h"

TAO_BEGIN_VERSIONED_NAMESPACE_DECL

class TAO_Root_POA;
class TAO_Acceptor;
class TAO_Profile;

/**
 * @class TAO_POA_Acceptor_Filter
 *
 * @brief POA Acceptor_Filter.
 *
 * A strategy for populating mprofile when only endpoints
 * provided in -ORBPOAEnpoints are included.
 */
class TAO_POAFiltering_Export TAO_POA_Acceptor_Filter
  : public TAO_Acceptor_Filter
{
public:

  TAO_POA_Acceptor_Filter (TAO_ORB_Core &oc,
                           TAO_Root_POA &poa);

  /// Populate @a mprofile with only endpoints provided in -ORBPOAEnpoints.
  int fill_profile (const TAO::ObjectKey &object_key,
                    TAO_MProfile &mprofile,
                    TAO_Acceptor **acceptors_begin,
                    TAO_Acceptor **acceptors_end,
                    CORBA::Short priority = TAO_INVALID_PRIORITY);

  /// Encodes the endpoints in the profiles into the TAO_TAG_ENDPOINTS
  /// tag component of profiles.
  int encode_endpoints (TAO_MProfile &mprofile);

private:

  /// Parses corbaloc address generated by Profile::to_string.
  int parse_corbaloc_address (TAO_Profile &profile,
                              ACE_CString &protocol,
                              TAO_EndpointSet &endpoints) const;

  /// Parses URL. 
  int parse_url_address (const char *ior,
                         ACE_CString &protocol,
                         TAO_EndpointSet &endpoints) const;

  /// Checks whether this profile has to be filtered or not.
  bool is_profile_filtered (TAO_Profile &profile,
                            const TAO_EndpointSet &poa_endpoints) const;

private:

  /// This is required to be able to access orb params.
  TAO_ORB_Core &orb_core_;

  /// POA is required by fill_profile.
  TAO_Root_POA &poa_;

};

TAO_END_VERSIONED_NAMESPACE_DECL

#include /**/ "ace/post.h"

#endif /* TAO_POA_ACCEPTOR_FILTER_H */
