#!/bin/sh
# $Id: build_thales_tao151.sh,v 1.5 2009-10-23 12:17:13 sma Exp $

#######################################################
#build.sh script should be used as follows:
#
#* Go to src directory of TAO distribution
#   cd <TAO distribution directory>/src
#* Untar sources
#   tar -xzf src.tar.gz
#* Run build.sh:
#   ./build.sh
#######################################################

if [ \( -d ace \) ]
then
  echo "Building with the existing extracted source files"
elif [ \( -r src.tar.gz \) ]
then
  echo "Extracting the source (one time only)"
  tar -xzf src.tar.gz
else
  echo "Can't find the \"src.tar.gz\" file to extract the source files"
  exit 0
fi

# 0 - Set evironment
ACE_ROOT=$PWD ;export ACE_ROOT
TAO_ROOT=$ACE_ROOT/TAO ;export TAO_ROOT
CIAO_ROOT=$TAO_ROOT/CIAO ;export CIAO_ROOT
if [ \( -z "$XERCESCROOT" \) ]
then
  echo "Environment variable XERCESCROOT needs to be set correctly prior to running this script."
  XERCESCROOT=/tools/exec/xerces-c-2_7_0 ;export XERCESCROOT
  echo "Assuming the correct location for XERCES is \"$XERCESCROOT\""
fi
if [ \( ! -d $XERCESCROOT/lib \) ]
then
  echo "XERCESC does not appear to be correctly installed at $XERCESCROOT"
  exit 0
fi
PATH=$ACE_ROOT/bin:$PATH ;export PATH
LD_LIBRARY_PATH=$ACE_ROOT/lib:$ACE_ROOT/ace:$XERCESCROOT/lib:$LD_LIBRARY_PATH ;export LD_LIBRARY_PATH

mkdir -p ../bin
mkdir -p ../lib

# 1 - Make the subset of TAO that is shipped
make -k -C ace
make -k -C apps/gperf
make -k -C ACEXML/common
make -k -C ACEXML/parser/parser
make -k -C ACEXML/apps/svcconf
make -k -C TAO/TAO_IDL
make -k -C TAO/tao

make -k -C TAO/orbsvcs/orbsvcs/MMPersistence
make -k -C TAO/orbsvcs/orbsvcs/MMPersistenceConfiguration
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.Svc_Utils
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosNaming_IDL
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosNaming
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosNaming_Skel
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosNaming_Serv
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosEvent_IDL
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosEvent
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosEvent_Skel
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosEvent_Serv
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosProperty_IDL
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosProperty
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosProperty_Skel
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.CosProperty_Serv
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.PortableGroup
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.FTORB_Utils
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.FT_ClientORB
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.IFRService
make -k -C TAO/orbsvcs/orbsvcs -f GNUmakefile.Security
make -k -C TAO/utils
make -k -C TAO/orbsvcs/Naming_Service
make -k -C TAO/orbsvcs/CosEvent_Service
make -k -C TAO/orbsvcs/IFR_Service -f GNUmakefile.IFR_Service
make -k -C TAO/orbsvcs/ImplRepo_Service
make -k -C TAO/orbsvcs/ImR_Service
make -k -C TAO/orbsvcs/IFR_Service -f GNUmakefile.TAO_IFR_BE
make -k -C TAO/orbsvcs/IFR_Service -f GNUmakefile.TAO_IFR_EXE
make -k -C TAO/orbsvcs/orbsvcs     -f GNUmakefile.SSLIOP
make -k -C TAO/CIAO

# ##########################
# Auxiliary functions
# ##########################

# copy file from a directory to another directory
dir2dircopy()
{
    from_dir=$1
    to_dir=$2

    while read file_from; do
	file_to=`echo $file_from | sed 's#^'${from_dir}'\(.*\)$#'${to_dir}'\1#g'`
	target_dir=`dirname $file_to`
	mkdir -p $target_dir
	cp -f $file_from $file_to 2>/dev/null
    done
}

#2 - Copy *.h; *.cpp; *.i; *.inl from ace/ (and all sub directories) into ../include/.
# preserving subdirectory structure.
# e.g. ./ace/config.h -> ../include/ace/config.h
#      ./ace/RMCast/RMCast_IO_UDP.i -> ../include/ace/RMCast/RMCast_IO_UDP.i
#      etc...
echo "Copying ../include/ace"
mkdir -p ../include
mkdir -p ../include/ace
find ace/ -name '*.h' -o -name '*.cpp' -o -name '*.i' -o -name '*.inl' | dir2dircopy ace/ ../include/ace/

#3a - Copy *.h; *.cpp; *.i; *.inl from TAO/tao (and all sub directories) into ../include/.
# preserving subdirectory structure
# e.g. ./TAO/tao/corba.h -> ../include/tao/corba.h
#      ./TAO/tao/Messaging/Asynch_Invocation_Adapter.cpp -> tao/Messaging/Asynch_Invocation_Adapter.cpp
#      ...etc...
echo "Copying ../include/tao"
mkdir -p ../include/tao
find TAO/tao/ -name '*.h' -o -name '*.cpp' -o -name '*.i' -o -name '*.inl' | dir2dircopy TAO/tao/ ../include/tao/

#3b - Copy *.h; *.cpp; *.i; *.inl from TAO/orbsvcs/orbsvcs (and all sub directories) into ../include/orbsvcs.
# preserving subdirectory structure
echo "Copying ../include/orbsvcs"
mkdir -p ../include/orbsvcs
find TAO/orbsvcs/orbsvcs/ -name '*.h' -o -name '*.cpp' -o -name '*.i' -o -name '*.inl' | dir2dircopy TAO/orbsvcs/orbsvcs/ ../include/orbsvcs/
rm -fr ../include/orbsvcs/AV
rm -fr ../include/orbsvcs/Concurrency
rm -fr ../include/orbsvcs/ESF
rm -fr ../include/orbsvcs/ETCL
rm -fr ../include/orbsvcs/Event
rm -fr ../include/orbsvcs/FtRtEvent
rm -fr ../include/orbsvcs/HTIOP
rm -fr ../include/orbsvcs/IFRService
rm -fr ../include/orbsvcs/LifeCycle
rm -fr ../include/orbsvcs/LoadBalancing
rm -fr ../include/orbsvcs/Log
rm -fr ../include/orbsvcs/Metrics
rm -fr ../include/orbsvcs/MMPersistence
rm -fr ../include/orbsvcs/MMPersistenceConfiguration
rm -fr ../include/orbsvcs/Notify
rm -fr ../include/orbsvcs/RTCosScheduling
rm -fr ../include/orbsvcs/Sched
rm -fr ../include/orbsvcs/Time
rm -fr ../include/orbsvcs/Trader

#3c - Copy *.h; *.cpp; *.i; *.inl from TAO/CIAO/ciao (and all sub directories) into ../include/ciao.
# preserving subdirectory structure
echo "Copying ../include/ciao"
mkdir -p ../include/ciao
find TAO/CIAO/ciao/ -name '*.h' -o -name '*.cpp' -o -name '*.i' -o -name '*.inl' | dir2dircopy TAO/CIAO/ciao/ ../include/ciao/

#3d - Copy *.h; *.cpp; *.i; *.inl from TAO/CIAO/DAnCE (and all sub directories) into ../include/DAnCE.
# preserving subdirectory structure
echo "Copying ../include/DAnCE"
mkdir -p ../include/DAnCE
find TAO/CIAO/DAnCE/ -name '*.h' -o -name '*.cpp' -o -name '*.i' -o -name '*.inl' | dir2dircopy TAO/CIAO/DAnCE/ ../include/DAnCE/
rm -rf ../include/DAnCE/tests

#3e - Copy *.h; *.cpp; *.i; *.inl from TAO/TAO_IDL/include (and all sub directories) into ../include/TAO_IDL.
# preserving subdirectory structure
echo "Copying ../include/TAO_IDL"
mkdir -p ../include/TAO_IDL
cp -f TAO/TAO_IDL/tao_idl.cpp ../include/TAO_IDL/
find TAO/TAO_IDL/include/ -name '*.h' -o -name '*.cpp' -o -name '*.i' -o -name '*.inl' | dir2dircopy TAO/TAO_IDL/include/ ../include/TAO_IDL/

#3f - Copy *.h; *.cpp; *.i; *.inl from TAO/TAO_IDL/be_include (and all sub directories) into ../include/TAO_IDL/be_include.
# preserving subdirectory structure
echo "Copying ../include/TAO_IDL/be_include"
mkdir -p ../include/TAO_IDL/be_include
find TAO/TAO_IDL/be_include/ -name 'be_extern.h' -o -name 'be_global.h' -o -name 'TAO_IDL_BE_Export.h' | dir2dircopy TAO/TAO_IDL/be_include/ ../include/TAO_IDL/be_include/

#3g - Copy *.h; *.cpp; *.i; *.inl from TAO/TAO_IDL/driver (and all sub directories) into ../include/TAO_IDL/driver.
# preserving subdirectory structure
echo "Copying ../include/TAO_IDL/driver"
mkdir -p ../include/TAO_IDL/driver
find TAO/TAO_IDL/driver/ -name '*.h' -o -name '*.cpp' -o -name '*.i' -o -name '*.inl' | dir2dircopy TAO/TAO_IDL/driver/ ../include/TAO_IDL/driver/

# 4 - Find and copy the libs from under ./ and copy them into ../lib
echo "Copying ../lib"
lib_list=`cat - <<EOF
  libACE.so.5.5.1
  libACEXML.so.5.5.1
  libACEXML_Parser.so.5.5.1
  libACEXML_XML_Svc_Conf_Parser.so.5.5.1
  libACE_SSL.so.5.5.1
  libTAO.so.1.5.1
  libTAO_AnyTypeCode.so.1.5.1
  libTAO_BiDirGIOP.so.1.5.1
  libTAO_CodecFactory.so.1.5.1
  libTAO_Codeset.so.1.5.1
  libTAO_CosEvent.so.1.5.1
  libTAO_CosEvent_Serv.so.1.5.1
  libTAO_CosEvent_Skel.so.1.5.1
  libTAO_CosNaming.so.1.5.1
  libTAO_CosNaming_Serv.so.1.5.1
  libTAO_CosNaming_Skel.so.1.5.1
  libTAO_CosProperty.so.1.5.1
  libTAO_CosProperty_Serv.so.1.5.1
  libTAO_CosProperty_Skel.so.1.5.1
  libTAO_CSD_Framework.so.1.5.1
  libTAO_CSD_ThreadPool.so.1.5.1
  libTAO_Domain.so.1.5.1
  libTAO_DynamicAny.so.1.5.1
  libTAO_DynamicInterface.so.1.5.1
  libTAO_FTORB_Utils.so.1.5.1
  libTAO_FT_ClientORB.so.1.5.1
  libTAO_IDL_BE.so.5.5.1
  libTAO_IDL_FE.so.5.5.1
  libTAO_IFRService.so.1.5.1
  libTAO_IFR_BE.so.1.5.1
  libTAO_IFR_Client.so.1.5.1
  libTAO_ImR.so.1.5.1
  libTAO_ImR_Activator.so.1.5.1
  libTAO_ImR_Activator_IDL.so.1.5.1
  libTAO_ImR_Client.so.1.5.1
  libTAO_ImR_Locator.so.1.5.1
  libTAO_ImR_Locator_IDL.so.1.5.1
  libTAO_IORInterceptor.so.1.5.1
  libTAO_IORManip.so.1.5.1
  libTAO_IORTable.so.1.5.1
  libTAO_Messaging.so.1.5.1
  libTAO_MMPersistConfig.so.1.5.1
  libTAO_MMPersistence.so.1.5.1
  libTAO_ObjRefTemplate.so.1.5.1
  libTAO_PI.so.1.5.1
  libTAO_PI_Server.so.1.5.1
  libTAO_PortableGroup.so.1.5.1
  libTAO_PortableServer.so.1.5.1
  libTAO_RTCORBA.so.1.5.1
  libTAO_RTPortableServer.so.1.5.1
  libTAO_RTScheduler.so.1.5.1
  libTAO_Security.so.1.5.1
  libTAO_SmartProxies.so.1.5.1
  libTAO_SSLIOP.so.1.5.1
  libTAO_Strategies.so.1.5.1
  libTAO_Svc_Utils.so.1.5.1
  libTAO_TypeCodeFactory.so.1.5.1
  libTAO_Utils.so.1.5.1
  libTAO_Valuetype.so.1.5.1
  libDAnCE_Cdmw_IDL.so.1.5.1
  libDAnCE_Deployment_stub.so.1.5.1
  libDAnCE_Deployment_svnt.so.1.5.1
  libDAnCE_DomainApplication.so.1.5.1
  libDAnCE_DomainApplicationManager.so.1.5.1
  libDAnCE_ExecutionManager.so.1.5.1
  libDAnCE_ExecutionManager_stub.so.1.5.1
  libDAnCE_NodeManager_stub.so.1.5.1
  libDAnCE_NodeApplication.so.1.5.1
  libDAnCE_NodeApplicationManager.so.1.5.1
  libDAnCE_NodeManager.so.1.5.1
  libDAnCE_Plan_Launcher.so.1.5.1
  libDAnCE_Plan_Launcher_Base.so.1.5.1
  libDAnCE_RedirectionService.so.1.5.1
  libCIAO_Client.so.1.5.1
  libCIAO_XML_Utils.so.5.5.1
  libXSC_Config_Handlers.so.1.5.1
  libXSC_Config_Handlers_Common.so.1.5.1
  libXSC_DynAny_Handler.so.1.5.1
  libXSC_XML_Handlers.so.1.5.1
  libDAnCE_Logger.so.1.5.1
  libPackage_Config_Handlers.so.1.5.1
  libCDMW_IDL3_TO_IDL2_BE.so.1.5.1
  libTAO_IDL3_TO_IDL2_BE.so.1.5.1
  libCIAO_IDL3_TO_XMI_BE.so.1.5.1
EOF
`
for lib_file in $lib_list; do
    find . -name $lib_file -exec cp '{}' ../lib ';'
done

# 5 - In ../lib create softlinks so that for any given libACE*.so.5.4.1 there exists a
# link libACE*.so that points to it.
# Similarly, for any given libTAO*.so.1.5.1 there exists a link libTAO*.so that points to it.
# See tao151/prism/install.sh for an example of how this could be done.
for lib_file in $lib_list; do
    if [ -f ../lib/$lib_file ]; then
	lib_link=`echo $lib_file | sed 's#\.so\..*#\.so#g'`
	ln -s -f ../lib/$lib_file ../lib/$lib_link
    fi
done

# 6a - Find and copy the excutables from under ./ and copy them into ../bin
echo "Copying ../bin"
bin_list=`cat <<EOF
  catior
  CosEvent_Service
  g++dep
  gperf
  IFR_Service
  ImplRepo_Service
  ImR_Activator
  mpc.pl
  mwc.pl
  Naming_Service
  nsadd
  nsdel
  nslist
  NT_Naming_Service
  tao_activator
  tao_activator_mgr
  tao_idl
  tao_ifr
  tao_imr
  tao_imrd
  tao_imrd_activator
  tao_imrd_mgr
  dance
  tao_idl3_to_idl2
  idl3_to_xmi
EOF
`
for bin_file in $bin_list; do
    find . \( -name $bin_file -a -type f \) -exec cp '{}' ../bin ';'
done

# 6b - copy * from bin/MakeProjectCreator (and all sub directories) into ../bin/MakeProjectCreator.
# preserving subdirectory structure
echo "Copying ../bin/MakeProjectCreator"
mkdir -p ../bin/MakeProjectCreator
find bin/MakeProjectCreator/ -name '*' | dir2dircopy bin/MakeProjectCreator/ ../bin/MakeProjectCreator/

# 6c - copy * from MPC (and all sub directories) into ../MPC.
# preserving subdirectory structure
echo "Copying ../MPC"
mkdir -p ../MPC
find MPC/ -name '*' | dir2dircopy MPC/ ../MPC/

# 7 - Copy ./include/makeinclude/* to ../include/makeinclude/.
echo "Copying ../include/makeinclude"
mkdir -p ../include/makeinclude
cp ./include/makeinclude/* ../include/makeinclude/.

# 8 - That's it
echo "Finished"
